package edu.rutgers.MOST.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

public class GDBBModel extends Model {

	protected Vector<Double> syntheticObjective;
	protected Vector<String> geneAssociations;
	protected Vector<String> distinctGeneAssociations;
	protected Vector<String> reactionAbbreviations;
	protected ArrayList<Map<Integer, Double>> gprMatrix;
	private double D;
	private double C;
	private double timeLimit;
	private int threadNum;
	private Vector<Double> syntheticObjectiveVector;
	private ReactionFactory rFactory;
	
	public double getD() {
		return D;
	}

	public void setD(double d) {
		D = d;
	}

	public double getC() {
		return C;
	}

	public void setC(double c) {
		C = c;
	}

	public GDBBModel(String columnName) {
		rFactory = new ReactionFactory("SBML");
		rFactory.setColumnName(columnName);
		
		this.geneAssociations = rFactory.getGeneAssociations();
		this.distinctGeneAssociations = rFactory.getUniqueGeneAssociations();		
		this.syntheticObjectiveVector = rFactory.getSyntheticObjectiveVector();
		this.reactionAbbreviations = rFactory.getReactionAbbreviations();
//		this.syntheticObjective = rFactory.getSyntheticObjectiveVector();
		
		//	GPR Matrix
		gprMatrix = new ArrayList<Map<Integer, Double>>();
//		for (int i = 0; i < distinctGeneAssociations.size(); i++) {
//			Map<Integer, Double> sRow = new HashMap<Integer, Double>();
//			gprMatrix.add(sRow);
//		}
		
		//	Populating Values
		for (int i = 0; i < distinctGeneAssociations.size(); i++) {
			gprMatrix.add(new HashMap<Integer, Double>());
			for(int j = 0; j < geneAssociations.size(); j++) {
				if(distinctGeneAssociations.elementAt(i).equals(geneAssociations.elementAt(j))) {
					gprMatrix.get(i).put(j, 1.0);
				}
			}
		}
		
		D = 100.0;
		C = 1;
		
		//	TODO Need to retrieve actual synthetic objective function
		
		syntheticObjective = new Vector<Double>();
//		for (int i = 0; i < reactions.size(); i++) {
//			SBMLReaction reac = (SBMLReaction) (reactions.elementAt(i));
//			if (reac.getId() != 729) {
//				syntheticObjective.add(0.0);
//			}
//			else {
////				System.out.println("Id = " + reac.getId() + ", Name = " + reac.getReactionName());
//				syntheticObjective.add(1.0);
//				break;
//			}
//		}
		
		for (int i = 0; i < reactions.size(); i++) {
//			SBMLReaction reac = (SBMLReaction) (reactions.elementAt(i));
			if (syntheticObjectiveVector.get(i) != 1.0) {
				syntheticObjective.add(0.0);
			}
			else {
				syntheticObjective.add(1.0);
			}
		}
	}

	public ReactionFactory getrFactory() {
		return rFactory;
	}

	public void setrFactory(ReactionFactory rFactory) {
		this.rFactory = rFactory;
	}
	
	public Vector<Double> getSyntheticObjective() {
		return syntheticObjective;
	}
	
	public void setSyntheticObjective( Vector< Double > syntheticObjective )
	{
		this.syntheticObjective = syntheticObjective;
	}

	public ArrayList<Map<Integer, Double>> getGprMatrix() {
		return gprMatrix;
	}

	public int getNumGeneAssociations() {
		return distinctGeneAssociations.size();
	}

	public void setTimeLimit(double timeLimit) {
		this.timeLimit = timeLimit;
	}

	public double getTimeLimit() {
		return timeLimit;
	}

	public int getThreadNum() {
		return threadNum;
	}

	public void setThreadNum(int threadNum) {
		this.threadNum = threadNum;
	}

	public Vector< String > getGeneAssociations()
	{
		return distinctGeneAssociations;
	}
}
